--
--  Copyright (C) 2025, AdaCore
--
--  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
--

package VSS.XML.Implementation.Template_Programs.Debug is

   procedure Dump_Program (Program : Instruction_Vectors.Vector);

end VSS.XML.Implementation.Template_Programs.Debug;
