--
--  Copyright (C) 2020, AdaCore
--
--  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
--

--  @private
--
--  Root of hierarchy of the internal implementation units. All code in this
--  hierarchy is subject to change. Applications MUST NOT use this package and
--  its children packages.

package VSS.JSON.Implementation is

   pragma Preelaborate;

end VSS.JSON.Implementation;
