--
--  Copyright (C) 2022, AdaCore
--
--  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
--

--  @private
--
--  Root package for internal implementation units of XML submodule.

package VSS.XML.Implementation is

   pragma Pure;

end VSS.XML.Implementation;
